@@ -38,6 +38,7 @@
 #include <glib/gstdio.h>
 
 #include "camel-groupwise-folder.h"
+#include "camel-groupwise-settings.h"
 #include "camel-groupwise-store-summary.h"
 #include "camel-groupwise-store.h"
 #include "camel-groupwise-summary.h"
@@ -58,9 +59,7 @@ const gchar * CREATE_CURSOR_VIEW = "peek id default recipient threading hasAttac
 
 struct _CamelGroupwiseStorePrivate {
 	gchar *server_name;
-	gchar *port;
 	gchar *user;
-	gchar *use_ssl;
 
 	gchar *base_url;
 
@@ -105,20 +104,35 @@ groupwise_auth_loop (CamelService *service,
 {
 	CamelSession *session = camel_service_get_session (service);
 	CamelStore *store = CAMEL_STORE (service);
+	CamelSettings *settings;
 	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (store);
 	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
+	CamelNetworkSecurityMethod security_method;
 	CamelURL *url;
 	gboolean authenticated = FALSE;
 	gchar *uri;
 	guint32 prompt_flags = CAMEL_SESSION_PASSWORD_SECRET;
+	const gchar *scheme;
 	EGwConnectionErrors errors = {E_GW_CONNECTION_STATUS_INVALID_OBJECT, NULL};
+	guint16 soap_port;
 
 	url = camel_service_get_camel_url (service);
+	settings = camel_service_get_settings (service);
 
-	if (priv->use_ssl && !g_str_equal (priv->use_ssl, "never"))
-		uri = g_strconcat ("https://", priv->server_name, ":", priv->port, "/soap", NULL);
+	security_method = camel_network_settings_get_security_method (
+		CAMEL_NETWORK_SETTINGS (settings));
+
+	soap_port = camel_groupwise_settings_get_soap_port (
+		CAMEL_GROUPWISE_SETTINGS (settings));
+
+	if (security_method == CAMEL_NETWORK_SECURITY_METHOD_NONE)
+		scheme = "http";
 	else
-		uri = g_strconcat ("http://", priv->server_name, ":", priv->port, "/soap", NULL);
+		scheme = "https";
+
+	uri = g_strdup_printf (
+		"%s://%s:%u/soap", scheme, priv->server_name, soap_port);
+
 	url->passwd = NULL;
 
 	while (!authenticated) {
@@ -130,7 +144,7 @@ groupwise_auth_loop (CamelService *service,
 				"GroupWise", url->user, url->host);
 			url->passwd =
 				camel_session_get_password (
-					session, service, "Groupwise", prompt,
+					session, service, prompt,
 					"password", prompt_flags, error);
 			g_free (prompt);
 
@@ -144,11 +158,7 @@ groupwise_auth_loop (CamelService *service,
 		}
 
 		priv->cnc = e_gw_connection_new_with_error_handler (uri, priv->user, url->passwd, &errors);
-		if (!E_IS_GW_CONNECTION(priv->cnc) && priv->use_ssl && g_str_equal (priv->use_ssl, "when-possible")) {
-			gchar *http_uri = g_strconcat ("http://", uri + 8, NULL);
-			priv->cnc = e_gw_connection_new (http_uri, priv->user, url->passwd);
-			g_free (http_uri);
-		}
+
 		if (!E_IS_GW_CONNECTION (priv->cnc)) {
 			if (errors.status == E_GW_CONNECTION_STATUS_INVALID_PASSWORD) {
 				/* We need to un-cache the password before prompting again */
@@ -179,16 +189,30 @@ check_for_connection (CamelService *service,
 {
 	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (service);
 	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
+	CamelSettings *settings;
 	struct addrinfo hints, *ai;
 	GError *local_error = NULL;
 
+	settings = camel_service_get_settings (service);
+
 	memset (&hints, 0, sizeof (hints));
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_family = PF_UNSPEC;
 	ai = camel_getaddrinfo(priv->server_name, "groupwise", &hints, cancellable, &local_error);
-	if (ai == NULL && priv->port != NULL && g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+	if (ai == NULL && g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+		gchar *port_string;
+		guint16 soap_port;
+
 		g_clear_error (&local_error);
-		ai = camel_getaddrinfo (priv->server_name, priv->port, &hints, cancellable, &local_error);
+
+		soap_port = camel_groupwise_settings_get_soap_port (
+			CAMEL_GROUPWISE_SETTINGS (settings));
+
+		port_string = g_strdup_printf ("%u", soap_port);
+		ai = camel_getaddrinfo (
+			priv->server_name, port_string, &hints,
+			cancellable, &local_error);
+		g_free (port_string);
 	}
 
 	if (ai == NULL) {
@@ -269,7 +293,7 @@ groupwise_connect_sync (CamelService *service,
 		store->summary, user_data_dir, '/');
 	camel_groupwise_store_summary_namespace_set (store->summary, ns);
 
-	if (camel_store_summary_count ((CamelStoreSummary *)store->summary) == 0) {
+	if (camel_store_summary_count ((CamelStoreSummary *) store->summary) == 0) {
 		/*Settting the refresh stamp to the current time*/
 		store->refresh_stamp = time (NULL);
 	}
@@ -292,7 +316,7 @@ groupwise_disconnect_cleanup (CamelService *service, gboolean clean, GError **er
 	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
 
 	if (groupwise_store->summary) {
-		camel_store_summary_save ((CamelStoreSummary *)groupwise_store->summary);
+		camel_store_summary_save ((CamelStoreSummary *) groupwise_store->summary);
 		g_object_unref (groupwise_store->summary);
 	}
 
@@ -454,8 +478,8 @@ groupwise_forget_folder (CamelGroupwiseStore *gw_store, const gchar *folder_name
 	g_rmdir (folder_dir);
 	g_free (folder_dir);
 
-	camel_store_summary_remove_path ( (CamelStoreSummary *)gw_store->summary, folder_name);
-	camel_store_summary_save ( (CamelStoreSummary *)gw_store->summary);
+	camel_store_summary_remove_path ( (CamelStoreSummary *) gw_store->summary, folder_name);
+	camel_store_summary_save ( (CamelStoreSummary *) gw_store->summary);
 
 	fi = groupwise_build_folder_info (gw_store, NULL, folder_name);
 	camel_store_folder_deleted (CAMEL_STORE (gw_store), fi);
@@ -570,7 +594,7 @@ groupwise_store_get_folder_sync (CamelStore *store,
 	}
 	g_free (folder_dir);
 
-	si = camel_store_summary_path ((CamelStoreSummary *)gw_store->summary, folder_name);
+	si = camel_store_summary_path ((CamelStoreSummary *) gw_store->summary, folder_name);
 	if (si) {
 		total = si->total;
 		camel_store_summary_info_free ((CamelStoreSummary *)(gw_store)->summary, si);
@@ -635,7 +659,7 @@ groupwise_store_get_folder_sync (CamelStore *store,
 				if (count == total || !list)
 						done = TRUE;
 
-				g_list_foreach (list, (GFunc)g_object_unref, NULL);
+				g_list_foreach (list, (GFunc) g_object_unref, NULL);
 				g_list_free (list);
 				list = NULL;
 				position = E_GW_CURSOR_POSITION_CURRENT;
@@ -692,7 +716,7 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 	}
 
 	if (!E_IS_GW_CONNECTION ( priv->cnc)) {
-		if (!groupwise_connect_sync (CAMEL_SERVICE ((CamelStore*)gw_store), cancellable, error)) {
+		if (!groupwise_connect_sync (CAMEL_SERVICE ((CamelStore*) gw_store), cancellable, error)) {
 			camel_service_unlock (CAMEL_SERVICE (gw_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 			return FALSE;
 		}
@@ -700,7 +724,7 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 
 	container_id =	g_strdup (g_hash_table_lookup (priv->name_hash, full_name));
 
-	si = camel_store_summary_path ((CamelStoreSummary *)gw_store->summary, name);
+	si = camel_store_summary_path ((CamelStoreSummary *) gw_store->summary, name);
 	if (si) {
 		total = si->total;
 		camel_store_summary_info_free ((CamelStoreSummary *)(gw_store)->summary, si);
@@ -772,7 +796,7 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 					if (count == total || !list)
 							done = TRUE;
 
-					g_list_foreach (list, (GFunc)g_object_unref, NULL);
+					g_list_foreach (list, (GFunc) g_object_unref, NULL);
 					g_list_free (list);
 					list = NULL;
 					position = E_GW_CURSOR_POSITION_CURRENT;
@@ -974,23 +998,23 @@ groupwise_folders_sync (CamelGroupwiseStore *store,
 	}
 
 	e_gw_connection_free_container_list (list);
-	count = camel_store_summary_count ((CamelStoreSummary *)store->summary);
+	count = camel_store_summary_count ((CamelStoreSummary *) store->summary);
 
-	count = camel_store_summary_count ((CamelStoreSummary *)store->summary);
+	count = camel_store_summary_count ((CamelStoreSummary *) store->summary);
 	for (i=0;i<count;i++) {
-		si = camel_store_summary_index ((CamelStoreSummary *)store->summary, i);
+		si = camel_store_summary_index ((CamelStoreSummary *) store->summary, i);
 		if (si == NULL)
 			continue;
 
 		info = g_hash_table_lookup (present, camel_store_info_path (store->summary, si));
 		if (info != NULL) {
-			camel_store_summary_touch ((CamelStoreSummary *)store->summary);
+			camel_store_summary_touch ((CamelStoreSummary *) store->summary);
 		} else {
-			camel_store_summary_remove ((CamelStoreSummary *)store->summary, si);
+			camel_store_summary_remove ((CamelStoreSummary *) store->summary, si);
 			count--;
 			i--;
 		}
-		camel_store_summary_info_free ((CamelStoreSummary *)store->summary, si);
+		camel_store_summary_info_free ((CamelStoreSummary *) store->summary, si);
 	}
 
 	g_hash_table_foreach (present, get_folders_free, NULL);
@@ -1025,21 +1049,21 @@ groupwise_get_folder_info_offline (CamelStore *store, const gchar *top,
 
 	path = gw_concat (name, "*");
 
-	for (i=0;i<camel_store_summary_count ((CamelStoreSummary *)groupwise_store->summary);i++) {
-		CamelStoreInfo *si = camel_store_summary_index ((CamelStoreSummary *)groupwise_store->summary, i);
+	for (i=0;i<camel_store_summary_count ((CamelStoreSummary *) groupwise_store->summary);i++) {
+		CamelStoreInfo *si = camel_store_summary_index ((CamelStoreSummary *) groupwise_store->summary, i);
 
 		if (si == NULL)
 			continue;
 
 		if ( !strcmp (name, camel_groupwise_store_info_full_name (groupwise_store->summary, si))
 		     || match_path (path, camel_groupwise_store_info_full_name (groupwise_store->summary, si))) {
-			fi = groupwise_build_folder_info (groupwise_store, NULL, camel_store_info_path ((CamelStoreSummary *)groupwise_store->summary, si));
+			fi = groupwise_build_folder_info (groupwise_store, NULL, camel_store_info_path ((CamelStoreSummary *) groupwise_store->summary, si));
 			fi->unread = si->unread;
 			fi->total = si->total;
 			fi->flags = si->flags;
 			g_ptr_array_add (folders, fi);
 		}
-		camel_store_summary_info_free ((CamelStoreSummary *)groupwise_store->summary, si);
+		camel_store_summary_info_free ((CamelStoreSummary *) groupwise_store->summary, si);
 	}
 
 	g_free (name);
@@ -1062,7 +1086,7 @@ groupwise_store_get_folder_info_sync (CamelStore *store,
 	/* Do not call groupwise_store_connected function as it would internall call folders_sync
 	   to populate the hash table which is used for mapping container id */
 	if (!(camel_offline_store_get_online (CAMEL_OFFLINE_STORE (store))
-	    && camel_service_connect_sync ((CamelService *)store, error)))
+	    && camel_service_connect_sync ((CamelService *) store, error)))
 		goto offline;
 
 	camel_service_lock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
@@ -1072,8 +1096,8 @@ groupwise_store_get_folder_info_sync (CamelStore *store,
 		return NULL;
 	}
 
-	camel_store_summary_touch ((CamelStoreSummary *)groupwise_store->summary);
-	camel_store_summary_save ((CamelStoreSummary *)groupwise_store->summary);
+	camel_store_summary_touch ((CamelStoreSummary *) groupwise_store->summary);
+	camel_store_summary_save ((CamelStoreSummary *) groupwise_store->summary);
 
 	camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
@@ -1103,7 +1127,7 @@ create_junk_folder (CamelStore *store)
 		status = e_gw_connection_modify_junk_settings (priv->cnc, JUNK_ENABLE, 0, 0,  JUNK_PERSISTENCE);
 	if (status == E_GW_CONNECTION_STATUS_OK) {
 		root = groupwise_build_folder_info (groupwise_store, parent_name, folder_name);
-		camel_store_summary_save ((CamelStoreSummary *)groupwise_store->summary);
+		camel_store_summary_save ((CamelStoreSummary *) groupwise_store->summary);
 
 		child_container_id = e_gw_connection_get_container_id (priv->cnc, "Junk Mail");
 		if (!child_container_id)
@@ -1173,7 +1197,7 @@ groupwise_store_create_folder_sync (CamelStore *store,
 		status = e_gw_connection_create_folder (priv->cnc,parent_id,folder_name, &child_container_id);
 	if (status == E_GW_CONNECTION_STATUS_OK) {
 		root = groupwise_build_folder_info (groupwise_store, parent_name,folder_name);
-		camel_store_summary_save ((CamelStoreSummary *)groupwise_store->summary);
+		camel_store_summary_save ((CamelStoreSummary *) groupwise_store->summary);
 
 		g_hash_table_insert (priv->id_hash, g_strdup (child_container_id), g_strdup (folder_name));
 		g_hash_table_insert (priv->name_hash, g_strdup (root->full_name), g_strdup (child_container_id));
@@ -1264,7 +1288,7 @@ groupwise_store_rename_folder_sync (CamelStore *store,
 	if (temp_new)
 		temp_new++;
 	else
-		temp_new = (gchar *)new_name;
+		temp_new = (gchar *) new_name;
 
 	if (!container_id || e_gw_connection_rename_folder (priv->cnc, container_id , temp_new) != E_GW_CONNECTION_STATUS_OK)
 	{
@@ -1375,18 +1399,22 @@ groupwise_store_get_trash_folder_sync (CamelStore *store,
 }
 
 static gboolean
-groupwise_can_refresh_folder (CamelStore *store, CamelFolderInfo *info, GError **error)
+groupwise_can_refresh_folder (CamelStore *store,
+                              CamelFolderInfo *info,
+                              GError **error)
 {
-	CamelURL *url;
-	gboolean res;
+	CamelService *service;
+	CamelSettings *settings;
+	gboolean check_all;
 
-	url = camel_service_get_camel_url (CAMEL_SERVICE (store));
+	service = CAMEL_SERVICE (store);
+	settings = camel_service_get_settings (service);
 
-	res = CAMEL_STORE_CLASS (camel_groupwise_store_parent_class)->
-		can_refresh_folder (store, info, error) ||
-		(camel_url_get_param (url, "check_all") != NULL);
+	check_all = camel_groupwise_settings_get_check_all (
+		CAMEL_GROUPWISE_SETTINGS (settings));
 
-	return res;
+	return CAMEL_STORE_CLASS (camel_groupwise_store_parent_class)->
+		can_refresh_folder (store, info, error) || check_all;
 }
 
 /*
@@ -1399,7 +1427,7 @@ camel_groupwise_store_connected (CamelGroupwiseStore *store,
                                  GError **error)
 {
 	if (camel_offline_store_get_online (CAMEL_OFFLINE_STORE (store))
-	    && camel_service_connect_sync ((CamelService *)store, error)) {
+	    && camel_service_connect_sync ((CamelService *) store, error)) {
 		CamelGroupwiseStore *gw_store = (CamelGroupwiseStore *) store;
 		CamelGroupwiseStorePrivate *priv = gw_store->priv;
 
@@ -1470,8 +1498,6 @@ groupwise_store_finalize (GObject *object)
 
 	g_free (groupwise_store->priv->user);
 	g_free (groupwise_store->priv->server_name);
-	g_free (groupwise_store->priv->port);
-	g_free (groupwise_store->priv->use_ssl);
 	g_free (groupwise_store->priv->base_url);
 	g_free (groupwise_store->root_container);
 
@@ -1498,7 +1524,6 @@ groupwise_store_initable_init (GInitable *initable,
 	CamelService *service;
 	CamelSession *session;
 	CamelURL *url;
-	const gchar *property_value;
 	const gchar *user_data_dir;
 	const gchar *store_uid;
 	gchar *transport_uid;
@@ -1527,8 +1552,8 @@ groupwise_store_initable_init (GInitable *initable,
 	path = g_alloca (strlen (user_data_dir) + 32);
 	sprintf (path, "%s/.summary", user_data_dir);
 	groupwise_store->summary = camel_groupwise_store_summary_new ();
-	camel_store_summary_set_filename ((CamelStoreSummary *)groupwise_store->summary, path);
-	camel_store_summary_touch ((CamelStoreSummary *)groupwise_store->summary);
+	camel_store_summary_set_filename ((CamelStoreSummary *) groupwise_store->summary, path);
+	camel_store_summary_touch ((CamelStoreSummary *) groupwise_store->summary);
 	camel_store_summary_load ((CamelStoreSummary *) groupwise_store->summary);
 
 	/*host and user*/
@@ -1540,19 +1565,6 @@ groupwise_store_initable_init (GInitable *initable,
 		url, CAMEL_URL_HIDE_PASSWORD |
 		CAMEL_URL_HIDE_PARAMS | CAMEL_URL_HIDE_AUTH);
 
-	/*soap port*/
-	property_value =  camel_url_get_param (url, "soap_port");
-	if (property_value == NULL)
-		groupwise_store->priv->port = g_strdup ("7191");
-	else if (strlen (property_value) == 0)
-		groupwise_store->priv->port = g_strdup ("7191");
-	else
-		groupwise_store->priv->port = g_strdup (property_value);
-
-	/*filter*/
-	if (camel_url_get_param (url, "filter"))
-		CAMEL_STORE (groupwise_store)->flags |= CAMEL_STORE_FILTER_INBOX;
-
 	/*Hash Table*/
 	groupwise_store->priv->id_hash =
 		g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
@@ -1561,10 +1573,6 @@ groupwise_store_initable_init (GInitable *initable,
 	groupwise_store->priv->parent_hash =
 		g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-	/*ssl*/
-	groupwise_store->priv->use_ssl =
-		g_strdup (camel_url_get_param (url, "use_ssl"));
-
 	CAMEL_STORE (groupwise_store)->flags &= ~CAMEL_STORE_VJUNK;
 	CAMEL_STORE (groupwise_store)->flags &= ~CAMEL_STORE_VTRASH;
 
@@ -1603,6 +1611,7 @@ camel_groupwise_store_class_init (CamelGroupwiseStoreClass *class)
 	object_class->finalize = groupwise_store_finalize;
 
 	service_class = CAMEL_SERVICE_CLASS (class);
+	service_class->settings_type = CAMEL_TYPE_GROUPWISE_SETTINGS;
 	service_class->get_name = groupwise_get_name;
 	service_class->connect_sync = groupwise_connect_sync;
 	service_class->disconnect_sync = groupwise_disconnect_sync;
@@ -1638,8 +1647,6 @@ camel_groupwise_store_init (CamelGroupwiseStore *groupwise_store)
 
 	d("in groupwise store init\n");
 	groupwise_store->priv->server_name = NULL;
-	groupwise_store->priv->port = NULL;
-	groupwise_store->priv->use_ssl = NULL;
 	groupwise_store->priv->user = NULL;
 	groupwise_store->priv->cnc = NULL;
 	groupwise_store->current_folder = NULL;
