@@ -50,6 +50,8 @@
 #include <proxy.h>
 #include <string.h>
 
+#include "camel-groupwise-settings.h"
+
 #define GW(name) e_builder_get_widget (priv->builder, name)
 
 #define ACCOUNT_PICTURE 0
@@ -461,12 +463,16 @@ proxy_dialog_initialize_widgets (EAccount *account)
 static EGwConnection *
 proxy_get_cnc (EAccount *account, GtkWindow *parent_window)
 {
+	CamelSettings *settings;
 	EGwConnection *cnc;
 	const gchar *failed_auth;
 	gchar *uri, *key, *prompt, *password = NULL;
 	CamelURL *url;
-	const gchar *poa_address, *use_ssl, *soap_port;
+	CamelNetworkSecurityMethod security_method;
+	const gchar *poa_address;
+	const gchar *scheme;
 	gboolean remember;
+	guint16 soap_port;
 
 	url = camel_url_new (account->source->url, NULL);
 	if (url == NULL)
@@ -475,17 +481,25 @@ proxy_get_cnc (EAccount *account, GtkWindow *parent_window)
 	if (!poa_address || strlen (poa_address) ==0)
 		return NULL;
 
-        soap_port = camel_url_get_param (url, "soap_port");
-	if (!soap_port || strlen (soap_port) == 0)
-                soap_port = "7191";
-	use_ssl = camel_url_get_param (url, "use_ssl");
+	settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+	camel_settings_load_from_url (settings, url);
+
+	g_object_get (
+		settings,
+		"soap-port", &soap_port,
+		"security-method", &security_method,
+		NULL);
+
+	g_object_unref (settings);
+
+	if (security_method == CAMEL_NETWORK_SECURITY_METHOD_NONE)
+		scheme = "http";
+	else
+		scheme = "https";
 
 	key =  g_strdup_printf ("groupwise://%s@%s/", url->user, poa_address);
 
-	if (use_ssl && !g_str_equal (use_ssl, "never"))
-		uri = g_strdup_printf ("https://%s:%s/soap", poa_address, soap_port);
-	else
-		uri = g_strdup_printf ("http://%s:%s/soap", poa_address, soap_port);
+	uri = g_strdup_printf ("%s://%s:%u/soap", scheme, poa_address, soap_port);
 
 	failed_auth = "";
 	cnc = NULL;
@@ -500,13 +514,9 @@ proxy_get_cnc (EAccount *account, GtkWindow *parent_window)
 	g_free (prompt);
 
 	cnc = e_gw_connection_new (uri, url->user, password);
-	if (!E_IS_GW_CONNECTION(cnc) && use_ssl && g_str_equal (use_ssl, "when-possible")) {
-		gchar *http_uri = g_strconcat ("http://", uri + 8, NULL);
-		cnc = e_gw_connection_new (http_uri, url->user, password);
-		g_free (http_uri);
-	}
 
 	camel_url_free (url);
+
 	return cnc;
 }
 
