@@ -35,6 +35,7 @@
 #include <mail/em-config.h>
 #include <mail/em-folder-properties.h>
 #include <mail/em-folder-tree.h>
+#include <mail/em-folder-tree-model.h>
 #include <mail/em-folder-selector.h>
 #include <mail/mail-mt.h>
 #include <mail/mail-vfolder.h>
@@ -68,9 +69,7 @@ static void
 refresh_folder_tree (EMFolderTreeModel *model, CamelStore *store)
 {
 	EAccount *account;
-	CamelProvider *provider;
 	const gchar *uid;
-	gchar *uri;
 
 	uid = camel_service_get_uid (CAMEL_SERVICE (store));
 	account = e_get_account_by_uid (uid);
@@ -78,16 +77,9 @@ refresh_folder_tree (EMFolderTreeModel *model, CamelStore *store)
 	if (account == NULL)
 		return;
 
-	uri = account->source->url;
 	em_folder_tree_model_remove_store (model, store);
 
-	provider = camel_provider_get (uri, NULL);
-	if (provider == NULL)
-		return;
-
-	if (!(provider->flags & CAMEL_PROVIDER_IS_STORAGE))
-		return;
-	em_folder_tree_model_add_store (model, store, account->name);
+	em_folder_tree_model_add_store (model, store);
 }
 
 void
@@ -149,10 +141,10 @@ create_folder_exec (struct _EMCreateFolder *m,
 	if ((m->fi = camel_store_create_folder_sync (
 		m->store, m->parent, m->name, cancellable, error))) {
 
-		if (camel_store_supports_subscriptions (m->store))
-			camel_store_subscribe_folder_sync (
-				m->store, m->full_name,
-				cancellable, error);
+		if (CAMEL_IS_SUBSCRIBABLE (m->store))
+			camel_subscribable_subscribe_folder_sync (
+				CAMEL_SUBSCRIBABLE (m->store),
+				m->full_name, cancellable, error);
 	}
 }
 
@@ -348,7 +340,7 @@ gw_new_shared_folder_cb (GtkAction *action, EShellView *shell_view)
 	shell_backend = e_shell_view_get_shell_backend (shell_view);
 
 	dialog = em_folder_selector_create_new (
-		parent, E_MAIL_BACKEND (shell_backend), 0,
+		parent, E_MAIL_BACKEND (shell_backend), em_folder_tree_model_get_default (), 0,
 		_("Create folder"), _("Specify where to create the folder:"));
 
 	selector = EM_FOLDER_SELECTOR (dialog);
@@ -422,44 +414,38 @@ org_gnome_shared_folder_factory (EPlugin *ep, EConfigHookItemFactoryData *hook_d
 EGwConnection *
 get_cnc (CamelStore *store)
 {
-		EGwConnection *cnc;
-		const gchar *uri, *property_value, *server_name, *user, *port;
-		gchar *use_ssl;
-		CamelService *service;
-		CamelURL *url;
+	const gchar *uri, *server_name, *user;
+	CamelService *service;
+	CamelSettings *settings;
+	CamelURL *url;
+	CamelNetworkSecurityMethod security_method;
+	const gchar *scheme;
+	guint16 soap_port;
 
-		if (!store)
-			return  NULL;
+	if (!store)
+		return  NULL;
 
-		service = CAMEL_SERVICE (store);
-		url = camel_service_get_camel_url (service);
-		server_name = g_strdup (url->host);
-		user = g_strdup (url->user);
-		property_value =  camel_url_get_param (url, "soap_port");
-		use_ssl = g_strdup (camel_url_get_param (url, "use_ssl"));
-		if (property_value == NULL)
-			port = g_strdup ("7191");
-		else if (strlen (property_value) == 0)
-			port = g_strdup ("7191");
-		else
-			port = g_strdup (property_value);
+	service = CAMEL_SERVICE (store);
+	url = camel_service_get_camel_url (service);
+	settings = camel_service_get_settings (service);
+	server_name = g_strdup (url->host);
+	user = g_strdup (url->user);
 
-		if (use_ssl && !g_str_equal (use_ssl, "never"))
-			uri = g_strconcat ("https://", server_name, ":", port, "/soap", NULL);
-		else
-			uri = g_strconcat ("http://", server_name, ":", port, "/soap", NULL);
+	g_object_get (
+		settings,
+		"soap-port", &soap_port,
+		"security-method", &security_method,
+		NULL);
 
-		cnc = e_gw_connection_new (uri, user, url->passwd);
-		if (!E_IS_GW_CONNECTION(cnc) && use_ssl && g_str_equal (use_ssl, "when-possible")) {
-			gchar *http_uri = g_strconcat ("http://", uri + 8, NULL);
-			cnc = e_gw_connection_new (http_uri, user, url->passwd);
-			g_free (http_uri);
-		}
-		g_free (use_ssl);
-		use_ssl = NULL;
+	if (security_method == CAMEL_NETWORK_SECURITY_METHOD_NONE)
+		scheme = "http";
+	else
+		scheme = "https";
 
-		return cnc;
+	uri = g_strdup_printf (
+		"%s://%s:%u/soap", scheme, server_name, soap_port);
 
+	return e_gw_connection_new (uri, user, url->passwd);
 }
 
 gchar *
